/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry;

import org.wso2.registry.utils.AuthorizationUtils;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class CollectionImpl extends ResourceImpl implements Collection {

    private int childCount;

    public CollectionImpl() {
    }

    public CollectionImpl(String[] paths) {
        content = paths;
    }

    public void setContent(Object content) {
        if (content instanceof String[] || content instanceof Resource[]) {
            super.setContent(content);
            return;
        }
        throw new IllegalArgumentException("Collections can only contain paths (String[])!");
    }

    public String[] getChildren() {
        return (String[]) content;
    }

    public String[] getChildren(int start, int pageLen) {
        if (content == null) {
            try {
                String sql = "SELECT R.RID, R.PATH  FROM RESOURCE R, DEPENDENCY D WHERE D.PARENT_RID=?" +
                        " AND D.CHILD_RID=R.RID ORDER BY R.PATH";
                PreparedStatement ps = dataSource.getConnection().prepareStatement(sql);
                ps.setString(1, getId());
                List<String> childPathList = new ArrayList();
                ResultSet results = ps.executeQuery();
                int current = 0;
                while (results.next()) {
                    String childID = results.getString("RID");
                    if (AuthorizationUtils.authorize(childID, ActionConstants.GET)) {
                        if (current >= start && (pageLen == -1 || current < start + pageLen))
                            childPathList.add(results.getString("PATH"));
                        current++;
                    }
                }
                if (current < start) {
                    throw new RegistryException("Didn't have enough results to start at #" + start);
                }
                ps.close();
                return childPathList.toArray(new String[childPathList.size()]);


            } catch (Exception e) {
                String msg = "Failed to get child paths of resource " +
                        getPath() + ". " + e.getMessage();
                //TODO need to log the message

            }
        } else {
            if ( content instanceof String[]){
                System.out.println("came here");
                List<String> childPathList = new ArrayList();
                String childNodes [] = (String[])content;
                int limit = 0;
                if (start > childNodes.length) {
                   return new String[0];
                }
                if (start + pageLen > childNodes.length) {
                    limit = childNodes.length;
                }
                for (int i = start ; i < limit; i++) {
                    String childNode = childNodes[i];
                    childPathList.add(childNode);
                }
                return childPathList.toArray(new String[childPathList.size()]);
            }
        }
        return new String[0];
    }

    public List<Resource> getChildResources() {
        return null;
    }


    public int getChildCount() {
        if (content == null) {
            String sql = "SELECT COUNT(*) AS CHILD_COUNT FROM DEPENDENCY WHERE PARENT_RID=?";
            try {
                PreparedStatement ps = dataSource.getConnection().prepareStatement(sql);
                ps.setString(1, getId());
                ResultSet results = ps.executeQuery();
                if (results.next()) {
                    childCount = results.getInt("CHILD_COUNT");
                    return childCount;
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }  else {
            if ( content instanceof String[]){
                return ((String[])content).length;
            }
        }
        return 0;
    }


    public void setChildCount(int count) {
        childCount = count;
    }
}
