/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry;

import java.util.Date;
import java.util.Properties;
import java.util.List;
import java.io.InputStream;

public interface Resource {

    String getId();

    String getAuthorUserName();

    Date getCreatedTime();

    Date getLastModified();

    String getDescription();

    void setDescription(String description);

    String getPath();

    /**
     * If resource is versioned, the associated version of the resource does not get modified by
     * any means. Therefore, the path of that version is the permanent path (permalink) of the
     * current state of the resource.
     *
     * @return Permanent path (permalink) of the resource.
     */
    String getPermanentPath();

    String getMediaType();

    int getState();

    void setMediaType(String mediaType);

    String getParentPath();

    String getProperty(String key);

    public List getPropertyValues(String key);

    Properties getProperties();

    void setProperty(String key, String value);

    void setProperty(String key, List<String> value);

    void addProperty(String key, String value);

    void setProperties(Properties properties);

    Object getContent() throws RegistryException;

    void setContent(Object content);

    String getLastUpdaterUserName();

    InputStream getContentStream() throws RegistryException;

    void setContentStream(InputStream contentStream);

    List<String> getAspects();

    void addAspect(String name);

    void removeAspect(String name);
}
