/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.dao;

import org.wso2.registry.Association;
import org.wso2.registry.RegistryException;
import org.wso2.registry.jdbc.utils.Transaction;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.sql.SQLException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;

public class AssociationDAO {

    private static final Log log = LogFactory.getLog(AssociationDAO.class);

    ResourceDAO resourceDAO = new ResourceDAO();

    public void addAssociation(String resourceID1 ,
                               String resourceID2 ,
                               String associationType) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String propSQL =
                    "INSERT INTO ASSOCIATION (RID1, RID2, ASSOCIATION_TYPE) VALUES (?,?,?)";
            PreparedStatement ps = conn.prepareStatement(propSQL);
            ps.setString(1, resourceID1);
            ps.setString(2, resourceID2);
            ps.setString(3, associationType);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to add association between resources " +
                    resourceID1 + " and " + resourceID2 + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }

    public void removeAssociation(String resourceID1 ,
                                  String resourceID2 ,
                                  String associationType) throws RegistryException {

        Connection conn = Transaction.getConnection();

        try {
            String propSQL =
                    "DELETE FROM ASSOCIATION WHERE RID1=? AND RID2=? AND ASSOCIATION_TYPE=?";
            PreparedStatement ps = conn.prepareStatement(propSQL);
            ps.setString(1, resourceID1);
            ps.setString(2,resourceID2);
            ps.setString(3,associationType);
            ps.executeUpdate();
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to remove association between resources " +
                    resourceID1 + " and " + resourceID2 + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }
    }



    public Association[] getAllAssociations(String resourcePath) throws RegistryException {

        String resourceID = resourceDAO.getResourceID(resourcePath);

        Connection conn = Transaction.getConnection();

        ArrayList arrayList = null;
        try {
            String propSQL = "SELECT RID2, ASSOCIATION_TYPE FROM ASSOCIATION WHERE RID1=?";
            PreparedStatement ps = conn.prepareStatement(propSQL);
            ps.setString(1, resourceID);
            ResultSet result = ps.executeQuery();
            arrayList = new ArrayList();

            while(result.next()) {
                Association association = new Association();
                association.setSourcePath(resourcePath);
                String destinationID = result.getString("RID2");
                association.setDestinationPath(resourceDAO.getResourcePath(destinationID));
                association.setAssociationType(result.getString("ASSOCIATION_TYPE"));
                arrayList.add(association);
            }
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to get all associations of resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        Association associations [] = new Association[arrayList.size()];
        for (int i = 0; i < arrayList.size(); i++) {
            associations[i] = (Association) arrayList.get(i);
        }
        return associations;
    }

    public Association[] getAllAssociationsForType(String resourcePath, String associationType)
            throws RegistryException {

        String resourceID = resourceDAO.getResourceID(resourcePath);

        Connection conn = Transaction.getConnection();

        ArrayList arrayList = null;
        try {
            String propSQL = "SELECT RID2, ASSOCIATION_TYPE FROM ASSOCIATION " +
                    "WHERE RID1=? AND ASSOCIATION_TYPE=?";
            PreparedStatement ps = conn.prepareStatement(propSQL);
            ps.setString(1,resourceID);
            ps.setString(2,associationType);
            ResultSet result = ps.executeQuery();
            arrayList = new ArrayList();

            while(result.next()) {
                Association association = new Association();
                association.setSourcePath(resourcePath);
                String destinationID = result.getString("RID2");
                association.setDestinationPath(resourceDAO.getResourcePath(destinationID));
                association.setAssociationType(associationType);
                arrayList.add(association);
            }
            ps.close();

        } catch (SQLException e) {

            String msg = "Failed to get associations of type " +
                    associationType + " for resource " + resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        Association associations [] = new Association[arrayList.size()];
        for (int i = 0; i < arrayList.size(); i++) {
            associations[i] = (Association) arrayList.get(i);
        }
        return associations;
    }
}
