/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers.builtin;

import org.wso2.registry.jdbc.handlers.Handler;
import org.wso2.registry.jdbc.handlers.RequestContext;
import org.wso2.registry.jdbc.Repository;
import org.wso2.registry.*;
import org.wso2.registry.users.UserRealm;

import javax.sql.DataSource;

public class Axis2RepositoryHandler extends Handler {

    public Axis2RepositoryHandler() {}

    public Axis2RepositoryHandler(Repository repository, Registry registry) {
        super(repository, registry);
    }

    public Resource get(RequestContext requestContext) throws RegistryException {
        return null;
    }

    public void put(RequestContext requestContext) throws RegistryException {

        String path = requestContext.getResourcePath();
        repository.put(path, requestContext.getResource());

        String confPath = path +
                          RegistryConstants.PATH_SEPARATOR + RegistryConstants
                .AXIS2_CONF_COLLECTION_NAME;
        CollectionImpl confCollection = new CollectionImpl();
        confCollection.setPath(confPath);
        confCollection.setMediaType(RegistryConstants.AXIS2_CONF_COLLECTION_MEDIA_TYPE);
        repository.put(confPath, confCollection);

        String servicesPath = path +
                              RegistryConstants.PATH_SEPARATOR + RegistryConstants
                .AXIS2_SERVICES_COLLECTION_NAME;
        CollectionImpl servicesCollection = new CollectionImpl();
        servicesCollection.setPath(servicesPath);
        servicesCollection.setMediaType(RegistryConstants.AXIS2_SERVICES_COLLECTION_MEDIA_TYPE);
        repository.put(servicesPath, servicesCollection);

        String modulesPath = path +
                             RegistryConstants.PATH_SEPARATOR + RegistryConstants
                .AXIS2_MODULES_COLLECTION_NAME;
        CollectionImpl modulesCollection = new CollectionImpl();
        modulesCollection.setPath(modulesPath);
        modulesCollection.setMediaType(RegistryConstants.AXIS2_MODULES_COLLECTION_MEDIA_TYPE);
        repository.put(modulesPath, modulesCollection);
    }

    public void importResource(RequestContext requestContext) throws RegistryException {
        
    }

    public void delete(RequestContext requestContext) throws RegistryException {
        requestContext.setProcessingComplete(false);
    }

    public void putChild(RequestContext requestContext) throws RegistryException {

        Resource childResource = requestContext.getResource();
        if (childResource == null) {
            childResource = repository.get(requestContext.getResourcePath());
            requestContext.setResource(childResource);
        }
        String childMediaType = requestContext.getResource().getMediaType();

        if(!(RegistryConstants.AXIS2_CONF_COLLECTION_MEDIA_TYPE.equals(childMediaType) ||
               RegistryConstants.AXIS2_SERVICES_COLLECTION_MEDIA_TYPE.equals(childMediaType) ||
               RegistryConstants.AXIS2_MODULES_COLLECTION_MEDIA_TYPE.equals(childMediaType))) {

            requestContext.setProcessingComplete(true);

        } else {
            String msg = "Resources of type: " + childMediaType +
                    " are not allowed to add as child resources of " +
                    "the typed collection Axis2 Repository.";
            throw new RegistryException(msg);
        }
    }

    public void importChild(RequestContext requestContext) throws RegistryException {
        
    }
}
