/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers.builtin;

import org.wso2.registry.jdbc.handlers.Handler;
import org.wso2.registry.jdbc.handlers.RequestContext;
import org.wso2.registry.jdbc.dao.RatingsDAO;
import org.wso2.registry.Resource;
import org.wso2.registry.RegistryException;
import org.wso2.registry.ResourceImpl;
import org.wso2.registry.RegistryConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Date;

public class RatingURLHandler extends Handler {

    private static final Log log = LogFactory.getLog(RatingURLHandler.class);

    public Resource get(RequestContext requestContext) throws RegistryException {

        String url = requestContext.getResourcePath();

        String[] parts = url.split(RegistryConstants.URL_SEPARATOR);

        if (parts.length != 2) {
            return null;
        }

        if (parts[0].length() == 0) {
            return null;
        }

        String[] details = parts[1].split(":");
        if (details.length != 2 || !details[0].equals("ratings") || details[1].length() == 0) {
            return null;
        }

        String resourcePath = parts[0];
        String ratedUserName = details[1];

        RatingsDAO ratingsDAO = new RatingsDAO();
        int rating = ratingsDAO.getRating(resourcePath, ratedUserName);
        Date ratedTime = ratingsDAO.getRatedTime(resourcePath, ratedUserName);

        ResourceImpl resource = new ResourceImpl();
        resource.setMediaType(RegistryConstants.RATING_MEDIA_TYPE);
        resource.setContent(rating);
        resource.setAuthorUserName(ratedUserName);
        resource.setPath(url);
        resource.setCreatedTime(ratedTime);
        resource.setLastModified(ratedTime);
        resource.addProperty("resourcePath", resourcePath);

        return resource;
    }

    public void put(RequestContext requestContext) throws RegistryException {
    }

    public void importResource(RequestContext requestContext) throws RegistryException {
    }

    public void delete(RequestContext requestContext) throws RegistryException {
    }

    public void putChild(RequestContext requestContext) throws RegistryException {
    }

    public void importChild(RequestContext requestContext) throws RegistryException {
    }
}
