/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers.builtin;

import org.wso2.registry.jdbc.handlers.Handler;
import org.wso2.registry.jdbc.handlers.RequestContext;
import org.wso2.registry.jdbc.dao.TagsDAO;
import org.wso2.registry.jdbc.dataobjects.TaggingDO;
import org.wso2.registry.Resource;
import org.wso2.registry.RegistryException;
import org.wso2.registry.ResourceImpl;
import org.wso2.registry.RegistryConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Date;

public class TagURLHandler extends Handler {

    private static final Log log = LogFactory.getLog(TagURLHandler.class);

    public Resource get(RequestContext requestContext) throws RegistryException {

        String url = requestContext.getResourcePath();

        String[] parts = url.split(RegistryConstants.URL_SEPARATOR);

        if (parts.length != 2) {
            return null;
        }

        if (parts[0].length() == 0) {
            return null;
        }

        String[] queries = parts[1].split(":");

        if (queries.length != 3) {
            return null;
        }

        String resourcePath = parts[0];
        String tagName = queries[1];
        String userName = queries[2];

        TagsDAO tagsDAO = new TagsDAO();
        TaggingDO taggingDO = tagsDAO.getTagging(resourcePath, tagName, userName);

        ResourceImpl resource = new ResourceImpl();
        resource.setMediaType(RegistryConstants.TAG_MEDIA_TYPE);
        resource.setPath(url);
        resource.setContent(taggingDO.getTagName());
        resource.setAuthorUserName(taggingDO.getTaggedUserName());
        final Date taggedTime = taggingDO.getTaggedTime();
        resource.setCreatedTime(taggedTime);
        resource.setLastModified(taggedTime);
        resource.addProperty("resourcePath", taggingDO.getResourcePath());

        requestContext.setProcessingComplete(true);

        return resource;
    }

    public void put(RequestContext requestContext) throws RegistryException {
    }

    public void importResource(RequestContext requestContext) throws RegistryException {
    }

    public void delete(RequestContext requestContext) throws RegistryException {
    }

    public void putChild(RequestContext requestContext) throws RegistryException {
    }

    public void importChild(RequestContext requestContext) throws RegistryException {
    }
}
