/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers.builtin;

import org.wso2.registry.jdbc.handlers.Handler;
import org.wso2.registry.jdbc.handlers.RequestContext;
import org.wso2.registry.jdbc.handlers.builtin.utils.WSDLFileProcessor;
import org.wso2.registry.jdbc.Repository;
import org.wso2.registry.Resource;
import org.wso2.registry.RegistryException;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.utils.AuthorizationUtil;
import org.wso2.registry.utils.RegistryUtils;

public class WSDLMediaTypeHandler extends Handler {

    private WSDLFileProcessor wsdlFileProcessor;

    public void setRepository(Repository repository) {
        super.setRepository(repository);

        wsdlFileProcessor = new WSDLFileProcessor(repository, this.registry);
    }

    public Resource get(RequestContext requestContext) throws RegistryException {
        return null;
    }

    public void put(RequestContext requestContext) throws RegistryException {
    }

    public void importResource(RequestContext requestContext) throws RegistryException {

        Resource metadata = requestContext.getResource();
        String parentPath = RegistryUtils.getParentPath(requestContext.getResourcePath());
        String sourceURL = requestContext.getSourceURL();

        String savedName = wsdlFileProcessor
                .saveWSDLFileToRegistry(sourceURL, parentPath, true, metadata);

        if (parentPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
            requestContext.setActualPath(parentPath + savedName);
        } else {
            requestContext.setActualPath(parentPath + RegistryConstants.PATH_SEPARATOR + savedName);
        }

        requestContext.setProcessingComplete(true);
    }

    public void delete(RequestContext requestContext) throws RegistryException {
    }

    public void putChild(RequestContext requestContext) throws RegistryException {
    }

    public void importChild(RequestContext requestContext) throws RegistryException {
    }
}
