/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.jdbc.realm.InMemoryRegistryRealm;
import org.wso2.registry.jdbc.realm.RegistryRealm;

public class CommentsTest extends TestCase {

    protected static Registry registry = null;
    protected static RegistryRealm realm = null;

    public void setUp() throws RegistryException {
        if (registry == null) {
            InMemoryEmbeddedRegistry embeddedRegistry = new InMemoryEmbeddedRegistry();
            registry = embeddedRegistry.getUserRegistry(
                    RegistryConstants.ADMIN_USER, RegistryConstants.ADMIN_PASSWORD);
        }
    }

    public void testCommentDelete() throws RegistryException {

        String r1Path = "/c1d1/c1";
        Collection r1 = registry.newCollection();
        registry.put(r1Path, r1);

        String c1Path = registry.addComment(r1Path, new Comment("test comment1"));
        String c2Path = registry.addComment(r1Path, new Comment("test comment2"));

        Comment[] comments1 = registry.getComments(r1Path);

        assertEquals("There should be two comments.", comments1.length, 2);

        String[] cTexts1 = {comments1[0].getText(), comments1[1].getText()};

        assertTrue("comment is missing", containsString(cTexts1, "test comment1"));
        assertTrue("comment is missing", containsString(cTexts1, "test comment2"));

        registry.delete(c1Path);

        Comment[] comments2 = registry.getComments(r1Path);

        assertEquals("There should be one comment.", 1, comments2.length);

        String[] cTexts2 = {comments2[0].getText()};

        assertTrue("comment is missing", containsString(cTexts2, "test comment2"));
        assertTrue("deleted comment still exists", !containsString(cTexts2, "test comment1"));
    }


    private boolean containsString(String[] array, String value) {

        boolean found = false;
        for (String anArray : array) {
            if (anArray.startsWith(value)) {
                found = true;
                break;
            }
        }

        return found;
    }
}
