package org.wso2.registry.web.utils;

import org.wso2.registry.Registry;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.Aspect;
import org.wso2.registry.jdbc.handlers.RequestContext;
import org.wso2.registry.web.UIConstants;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;

/**
 * TODO: JavaDoc
 */
public class LifecycleUtil {
    public static class SimpleLifecycle extends Aspect {
        public static final String STATE_PROP = "SimpleLifecycle.state";
        public static final String INIT = "init";
        public static final String FINAL = "final";

        public static final String ACTION = "transition";

        public void associate(Resource resource, Registry registry) throws RegistryException {
            resource.setProperty(STATE_PROP, INIT);
        }

        public void transition(RequestContext context, String action) throws RegistryException {
            if (!ACTION.equals(action)) {
                throw new RegistryException("Wrong action");
            }

            Resource r = context.getResource();
            String state = r.getProperty(STATE_PROP);
            if (state == null) {
                throw new RegistryException("No state property");
            }

            if (!INIT.equals(state)) {
                throw new RegistryException("Invalid state '" + state + "'");
            }

            r.setProperty(STATE_PROP, FINAL);
        }

        static String [] actions = new String [] { ACTION };

        public String [] getAvailableActions(RequestContext context) {
            Resource r = context.getResource();
            String state = r.getProperty(STATE_PROP);
            if (INIT.equals(state)) return actions;
            return null;
        }
    }
    public static void addAspect(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        try {
            String resourcePath = request.getParameter("resourcePath");
            String aspectText = request.getParameter("aspect");
            Registry registry = CommonUtil.getUserRegistry(request);
            List<String> aspectList = new ArrayList();
            Map<String, String[]> lifecycles = new HashMap<String, String[]>();


            registry.addAspect(aspectText,new SimpleLifecycle());
            registry.associateAspect(resourcePath,aspectText);
            Resource resource = registry.get(resourcePath);
            aspectList = resource.getAspects();
            registry.put(resourcePath,resource);

            for(String asp:aspectList){
                String[] actions=registry.getAspectActions(resourcePath,asp);
                lifecycles.put(asp,actions);
            }

            request.getSession().setAttribute(UIConstants.AJAX_LIFECYCLE_LIST, lifecycles);

        } catch (RegistryException e) {
            request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, e.getMessage());
        }
        request.getRequestDispatcher(UIConstants.AJAX_LIFECYCLELIST_JSP).forward(request, response);
    }
}
